@@ -15,6 +15,7 @@ from pytypeclass.nonempty_list import NonemptyList
 
 from dollar_lambda.error import (
     ArgumentError,
+    BinaryError,
     HelpError,
     MissingError,
     UnequalError,
@@ -49,10 +50,16 @@ class Parse(Generic[A_co]):
     unparsed: Sequence[str]
 
 
+@dataclass
+class SuccessError(ArgumentError, Generic[A]):
+    input: Sequence[str]
+    output: NonemptyList[Parse[A]]
+
+
 def empty() -> "Parser[Sequence]":
     """
     Always returns {}, no matter the input. Mostly useful for use in `nonpositional`.
-    >>> empty().parse_args("any", "arguments")
+    >>> empty().parse_args("any", "arguments", allow_unparsed=True)
     {}
     """
     return Parser[Sequence[A]].empty()
@@ -69,20 +76,12 @@ def binary_usage(a: Optional[str], op: str, b: Optional[str], add_brackets=True)
     return usage or None
 
 
-__pdoc__ = {}
-
-
 @dataclass
 class Parser(MonadPlus[A_co]):
     """
     Main class powering the argument parser.
     """
 
-    __pdoc__["Parser.__add__"] = True
-    __pdoc__["Parser.__or__"] = True
-    __pdoc__["Parser.__rshift__"] = True
-    __pdoc__["Parser.__ge__"] = True
-
     f: Callable[[Sequence[str]], Result[Parse[A_co]]]
     usage: Optional[str]
     helps: Dict[str, str]
@@ -121,6 +120,9 @@ class Parser(MonadPlus[A_co]):
         usage = binary_usage(self.usage, " ", other.usage, add_brackets=False)
         return replace(p, usage=usage)
 
+    def __ge__(self, f: Callable[[A_co], Monad[B]]) -> "Parser[B]":
+        return self.bind(f)
+
     def __or__(  # type: ignore[override]
         self: "Parser[A_co]",
         other: "Parser[B]",
@@ -135,16 +137,16 @@ class Parser(MonadPlus[A_co]):
         >>> p.parse_args("--verbose")
         {'verbose': True}
 
-        Note that when both arguments are supplied, this will only parse the first:
-        >>> p.parse_args("--verbose", "--option", "x")
-        {'verbose': True}
+        Note that by default, `parse_args` adds `>> done` to the end of parsers, causing
+        `p` to fail when both arguments are supplied:
 
-        If you want this to fail, use `>>` (`Parser.__rshift__`) with `done()` or another parser:
-        >>> (p >> done()).parse_args("--verbose", "--option", "x")
+        >>> p.parse_args("--verbose", "--option", "x")
         usage: [--option OPTION | --verbose]
         Unrecognized argument: --option
-        >>> p.parse_args("--option", "x")
-        {'option': 'x'}
+
+        To disable this behavior, use `allow_unparsed`:
+        >>> p.parse_args("--verbose", "--option", "x", allow_unparsed=True)
+        {'verbose': True}
         """
 
         def f(cs: Sequence[str]) -> Result[Parse["A_co | B"]]:
@@ -157,7 +159,7 @@ class Parser(MonadPlus[A_co]):
         )
 
     def __rshift__(
-        self: "Parser[Sequence[A]]", p: "Parser[Sequence[B]]"
+        self: "Parser[typing.Sequence[A]]", p: "Parser[typing.Sequence[B]]"
     ) -> "Parser[Sequence[A | B]]":
         """
         This applies parsers in sequence. If the first parser succeeds, the unparsed remainder
@@ -174,22 +176,74 @@ class Parser(MonadPlus[A_co]):
         usage: FIRST SECOND
         The following arguments are required: second
         """
-        # def f(p1: Sequence[D]) -> Parser[Parse[Sequence[D | B]]]:
-        #     def g(p2: Sequence[B]) -> Parser[Sequence[D | B]]:
-        #         return Parser.return_(p1 + p2)
 
-        #     return p >= g
+        def f(p1: typing.Sequence[A]) -> Parser[Sequence[A | B]]:
+            def g(p2: typing.Sequence[B]) -> Parser[Sequence[A | B]]:
+                _p1 = p1 if isinstance(p1, Sequence) else Sequence(p1)
+                _p2 = p2 if isinstance(p2, Sequence) else Sequence(p2)
+                return Parser.return_(_p1 + _p2)
 
-        # return self >= f
-        parser = self >= (lambda p1: (p >= (lambda p2: Parser.return_(p1 + p2))))
+            return p >= g
+
+        parser = self >= f
+        # parser = self >= (lambda p1: (p >= (lambda p2: Parser.return_(p1 + p2))))
         return replace(
             parser,
             usage=binary_usage(self.usage, " ", p.usage, add_brackets=False),
             helps={**self.helps, **p.helps},
         )
 
-    def __ge__(self, f: Callable[[A_co], Monad[B]]) -> "Parser[B]":
-        return self.bind(f)
+    def __xor__(
+        self: "Parser[Sequence[A]]", other: "Parser[Sequence[B]]"
+    ) -> "Parser[Sequence[A | B]]":
+        """
+        This is the same as `__or__`, but it succeeds only if one of the two parsers fails.
+
+        >>> p = argument("int", type=int) ^ argument("div", type=lambda x: 1 / float(x))
+        >>> p.parse_args("inf")  # succeeds because int("inf") fails
+        {'div': 0.0}
+        >>> p.parse_args("0")  # succeeds because 1 / 0 throws an error
+        {'int': 0}
+        >>> p.parse_args("1")  # fails because both parsers succeed
+        Both parsers succeeded. This causes ^ to fail.
+        """
+        p = (self.fails() >> other) | (other.fails() >> self)
+
+        def f(error: ArgumentError) -> ArgumentError:
+            if isinstance(error, BinaryError):
+                return ArgumentError("Both parsers succeeded. This causes ^ to fail.")
+            return error
+
+        return p.map_error(f)
+
+    def apply(self: "Parser[A]", f: Callable[[A], Result[B]]) -> "Parser[B]":  # type: ignore[misc]
+        """
+        Takes the output of `parser` and applies `f` to it. Convert any errors that arise into `ArgumentError`.
+
+        >>> p1 = flag("hello")
+        >>> p1.parse_args("--hello", return_dict=False)
+        [('hello', True)]
+
+        This will double `p1`'s output:
+        >>> p2 = p1.apply(lambda kv: Result.return_(kv + kv))
+        >>> p2.parse_args("--hello", return_dict=False)
+        [('hello', True), ('hello', True)]
+        """
+
+        def g(a: A) -> Parser[B]:
+            try:
+                y = f(a)
+            except Exception as e:
+                usage = f"An argument {a}: raised exception {e}"
+                y = Result(ArgumentError(usage))
+            return Parser(
+                lambda cs: y >= (lambda parsed: Result.return_(Parse(parsed, cs))),
+                usage=self.usage,
+                helps=self.helps,
+            )
+
+        p = self >= g
+        return replace(p, usage=self.usage, helps=self.helps)
 
     def bind(self, f: Callable[[A_co], Monad[B]]) -> "Parser[B]":
         """
@@ -235,11 +289,34 @@ class Parser(MonadPlus[A_co]):
     def empty(cls: Type["Parser[Sequence[A]]"]) -> "Parser[Sequence[A]]":
         """
         Always returns {}, no matter the input. Mostly useful for use in `nonpositional`.
-        >>> empty().parse_args("any", "arguments")
+        >>> empty().parse_args("any", "arguments", allow_unparsed=True)
         {}
         """
         return cls.return_(Sequence([]))
 
+    def fails(self: "Parser[Sequence[A]]") -> "Parser[Sequence[A]]":
+        """
+        Succeeds only if self fails. Does not consume any input.
+
+        >>> flag("x").fails().parse_args("not x", allow_unparsed=True)  # succeeds
+        {}
+        >>> flag("x").fails().parse_args("-x", allow_unparsed=True)  # fails
+        Parser unexpectedly succeeded.
+        """
+
+        def g(cs: Sequence[str]) -> Result[Parse[Sequence[A]]]:
+            parse = self.parse(cs).get
+            if isinstance(parse, Exception):
+                return Result.return_(Parse(Sequence([]), cs))
+            else:
+                return Result.zero(
+                    error=SuccessError(
+                        "Parser unexpectedly succeeded.", input=cs, output=parse
+                    )
+                )
+
+        return Parser(g, usage=None, helps=self.helps)
+
     def handle_error(self, error: ArgumentError) -> None:
         def print_usage(usage: str):
             print("usage:", end="\n" if "\n" in usage else " ")
@@ -317,14 +394,24 @@ class Parser(MonadPlus[A_co]):
             helps=self.helps,
         )
 
+    def map_error(self, f: Callable[[ArgumentError], ArgumentError]) -> "Parser[A_co]":
+        def g(cs: Sequence[str]) -> Result[Parse[A_co]]:
+            parse = self.parse(cs)
+            if isinstance(parse.get, ArgumentError):
+                return Result.zero(error=f(parse.get))
+            else:
+                return parse
+
+        return Parser(g, usage=None, helps=self.helps)
+
     def optional(self: "Parser[Sequence[A]]") -> "Parser[Sequence[A]]":
         """
         Allows arguments to be optional:
-        >>> p1 = flag("optional") >> done()
+        >>> p1 = flag("optional")
         >>> p = p1.optional()
         >>> p.parse_args("--optional")
         {'optional': True}
-        >>> p.parse_args("--misspelled")  # succeeds with no output
+        >>> p.parse_args("--misspelled", allow_unparsed=True)  # succeeds with no output
         {}
         >>> p1.parse_args("--misspelled")
         usage: --optional
@@ -341,8 +428,9 @@ class Parser(MonadPlus[A_co]):
     def parse_args(
         self: "Parser[Sequence[KeyValue]]",
         *args: str,
-        return_dict: bool = True,
+        allow_unparsed: bool = False,
         check_help: bool = True,
+        return_dict: bool = True,
     ) -> "typing.Sequence[KeyValueTuple] | Dict[str, Any]":
         """
         The main way the user extracts parsed results from the parser.
@@ -351,12 +439,15 @@ class Parser(MonadPlus[A_co]):
         ----------
         args : str
             A sequence of strings to parse. If empty, defaults to `sys.argv[1:]`.
-        return_dict : bool
-            Returns a sequence of tuples instead of dictionary, thereby allowing duplicate keys.
-            The tuples are `KeyValueTuple` namedtuples, with fields `key` and `value`.
+        allow_unparsed : bool
+            Whether to cause parser to fail if there are unparsed inputs. Note that setting this to false
+            may cause unexpected behavior when using `nonpositional` or `Args`.
         check_help : bool
             Before running the parser, checks if the input string is `--help` or `-h`.
             If it is, returns the usage message.
+        return_dict : bool
+            Returns a sequence of tuples instead of dictionary, thereby allowing duplicate keys.
+            The tuples are `KeyValueTuple` namedtuples, with fields `key` and `value`.
 
         Examples
         --------
@@ -367,9 +458,19 @@ class Parser(MonadPlus[A_co]):
         usage: A
         """
         _args = args if args or TESTING else sys.argv[1:]
+        if not allow_unparsed:
+            return (self >> done()).parse_args(
+                *_args,
+                allow_unparsed=True,
+                check_help=check_help,
+                return_dict=return_dict,
+            )
         if check_help:
-            return wrap_help(self).parse_args(
-                *_args, return_dict=return_dict, check_help=False
+            return self.wrap_help().parse_args(
+                *_args,
+                allow_unparsed=allow_unparsed,
+                return_dict=return_dict,
+                check_help=False,
             )
         result = self.parse(Sequence(list(_args))).get
         if isinstance(result, ArgumentError):
@@ -402,6 +503,97 @@ class Parser(MonadPlus[A_co]):
 
         return Parser(f, usage=None, helps={})
 
+    def sat(
+        self: "Parser[A]",
+        predicate: Callable[[A], bool],
+        on_fail: Callable[[A], ArgumentError],
+    ) -> "Parser[A]":
+        """
+        Applies `parser`, applies a predicate to the result and fails if this returns false.
+
+        >>> p = option("x", type=int).many().sat(
+        ...     lambda kvs: sum([kv.value for kv in kvs]) > 0,
+        ...     lambda x: ArgumentError(f"The values in {list(x)} must sum to more than 0."),
+        ... )
+        >>> p.parse_args("-x", "-1", "-x", "1")  # fails
+        usage: [-x X ...]
+        The values in [KeyValue(key='x', value=-1), KeyValue(key='x', value=1)] must sum to more than 0.
+
+        >>> p.parse_args("-x", "-1", "-x", "2")  # succeeds
+        {'x': 2}
+
+        Parameters
+        ----------
+        parser : Parser[A]
+            The parser to apply.
+        predicate : Callable[[A], bool]
+            The predicate to apply to the result of `parser`. `sat` fails if this predicate returns false.
+        on_fail : Callable[[A], ArgumentError]
+            A function producing an ArgumentError to return if the predicate fails.
+            Takes the output of `parser` as an argument.
+        """
+
+        def f(x: A) -> Result[A]:
+            return Result(NonemptyList(x) if predicate(x) else on_fail(x))
+
+        return self.apply(f)
+
+    def type(
+        self: "Parser[Sequence[KeyValue[str]]]", f: Callable[[str], Any]
+    ) -> "Parser[Sequence[KeyValue[Any]]]":
+        """
+        A wrapper around `apply` that simply applies `f` to the value of the most recently parsed input.
+        >>> p1 = option("x") >> option("y")
+        >>> p = p1.type(int)
+        >>> p.parse_args("-x", "1", "-y", "2")  # converts "1" but not "2"
+        {'y': '2', 'x': 1}
+        """
+
+        def g(
+            kvs: Sequence[KeyValue[str]],
+        ) -> Result[Sequence[KeyValue[Any]]]:
+            head, *tail = kvs.get
+            try:
+                y = f(head.value)
+            except Exception as e:
+                usage = f"argument {head.value}: raised exception {e}"
+                return Result(ArgumentError(usage))
+            head = replace(head, value=y)
+            return Result.return_(Sequence([*tail, head]))
+
+        p = self.apply(g)
+        return replace(p, usage=self.usage, helps=self.helps)
+
+    def wrap_error(self, error: ArgumentError) -> "Parser[A_co]":
+        return self.map_error(lambda _: error)
+
+    def wrap_help(self: "Parser[A]") -> "Parser[A]":
+        """
+        This checks for the `--help` or `-h` flag before applying `parser`.
+        If either of the flags is present, returns the usage message for `parser`.
+
+        >>> p = flag("help", help="Print this help message.").wrap_help()
+        >>> p.parse_args("--help")
+        usage: --help
+        help: Print this help message.
+        >>> p.parse_args("-h")
+        usage: --help
+        help: Print this help message.
+
+        We can use `wrap_help` to print partial usage messages, e.g. for subcommands:
+        >>> subcommand1 = equals("subcommand1") >> option("option1").wrap_help()
+        >>> subcommand2 = equals("subcommand2") >> option("option2").wrap_help()
+        >>> p = subcommand1 | subcommand2
+        >>> p.parse_args("subcommand1", "-h")
+        usage: --option1 OPTION1
+        >>> p.parse_args("subcommand2", "-h")
+        usage: --option2 OPTION2
+        """
+        _help_parser: Parser[Sequence[A]] = help_parser(self.usage, Sequence([]))
+
+        p = _help_parser >= (lambda _: self)
+        return replace(p, usage=self.usage, helps=self.helps)
+
     @classmethod
     def zero(cls, error: Optional[ArgumentError] = None) -> "Parser[A_co]":
         """
@@ -422,41 +614,9 @@ class Parser(MonadPlus[A_co]):
         return Parser(lambda _: Result.zero(error=error), usage=None, helps={})
 
 
-def apply(
-    f: Callable[[A], Result[B]], parser: Parser[A]  # type: ignore[misc]
-) -> Parser[B]:
+def apply(f: Callable[[str], B], description: str) -> Parser[B]:
     """
-    Takes the output of `parser` and applies `f` to it. Convert any errors that arise into `ArgumentError`.
-
-    >>> p1 = flag("hello")
-    >>> p1.parse_args("--hello", return_dict=False)
-    [('hello', True)]
-
-    This will double `p1`'s output:
-    >>> p2 = apply(lambda kv: Result.return_(kv + kv), p1)
-    >>> p2.parse_args("--hello", return_dict=False)
-    [('hello', True), ('hello', True)]
-    """
-
-    def g(a: A) -> Parser[B]:
-        try:
-            y = f(a)
-        except Exception as e:
-            usage = f"An argument {a}: raised exception {e}"
-            y = Result(ArgumentError(usage))
-        return Parser(
-            lambda cs: y >= (lambda parsed: Result.return_(Parse(parsed, cs))),
-            usage=parser.usage,
-            helps=parser.helps,
-        )
-
-    p = parser >= g
-    return replace(p, usage=parser.usage, helps=parser.helps)
-
-
-def apply_item(f: Callable[[str], B], description: str) -> Parser[B]:
-    """
-    A shortcut for `apply(f, item(description))`
+    A shortcut for `item(description).apply(f)`
     and spares `f` the trouble of outputting a `Result` object.
 
     >>> p1 = argument("foo")
@@ -464,7 +624,7 @@ def apply_item(f: Callable[[str], B], description: str) -> Parser[B]:
     [('foo', 'bar')]
 
     Here we use `f` to directly manipulate the binding generated by `item`:
-    >>> p2 = apply_item(lambda bar: [KeyValue(bar + "e", bar + "f")], description="baz")
+    >>> p2 = apply(lambda bar: [KeyValue(bar + "e", bar + "f")], description="baz")
     >>> p2.parse_args("bar", return_dict=False)
     [('bare', 'barf')]
     """
@@ -478,7 +638,7 @@ def apply_item(f: Callable[[str], B], description: str) -> Parser[B]:
             return Result(ArgumentError(usage))
         return Result.return_(y)
 
-    return apply(g, item(description))
+    return item(description).apply(g)
 
 
 def argument(
@@ -508,7 +668,7 @@ def argument(
     parser = item(dest)
     _type: Callable[[str], Any] = str if type is None else type  # type: ignore[assignment]
     if _type is not str:
-        parser = type_(_type, parser)
+        parser = parser.type(_type)
     helps = {dest: help} if help else {}
     parser = replace(parser, usage=dest.upper(), helps=helps)
     return parser
@@ -552,9 +712,20 @@ def done() -> Parser[Sequence[A]]:
 
     Without `done` the parser will not complain about leftover (unparsed) input:
 
-    >>> flag("verbose").parse_args("--verbose", "--quiet")
+    >>> flag("verbose").parse_args("--verbose", "--quiet", allow_unparsed=True)
     {'verbose': True}
 
+    When `allow_unparsed=False` (the default), `parse_args` adds `>> done()`
+    to the end of the parser:
+
+    >>> flag("verbose").parse_args("--verbose", "--quiet", allow_unparsed=False)
+    usage: --verbose
+    Unrecognized argument: --quiet
+
+    >>> (flag("verbose") >> done()).parse_args("--verbose", "--quiet", allow_unparsed=True)
+    usage: --verbose
+    Unrecognized argument: --quiet
+
     `--quiet` is not parsed here but this does not cause the parser to fail.
     If we want to prevent leftover inputs, we can use `done`:
 
@@ -623,7 +794,7 @@ def equals(s: str, peak=False) -> Parser[Sequence[KeyValue[str]]]:
             name=s,
         )
     else:
-        return sat_item(
+        return sat(
             predicate=lambda _s: _s == s,
             on_fail=lambda _s: UnequalError(
                 left=s, right=_s, usage=f"Expected '{s}'. Got '{_s}'"
@@ -736,34 +907,6 @@ def help_parser(usage: Optional[str], parsed: A) -> Parser[A]:
     return Parser(f, usage=None, helps={})
 
 
-def wrap_help(parser: Parser[A]) -> Parser[A]:
-    """
-    This checks for the `--help` or `-h` flag before applying `parser`.
-    If either of the flags is present, returns the usage message for `parser`.
-
-    >>> p = wrap_help(flag("help", help="Print this help message."))
-    >>> p.parse_args("--help")
-    usage: --help
-    help: Print this help message.
-    >>> p.parse_args("-h")
-    usage: --help
-    help: Print this help message.
-
-    We can use `wrap_help` to print partial usage messages, e.g. for subcommands:
-    >>> subcommand1 = equals("subcommand1") >> wrap_help(option("option1"))
-    >>> subcommand2 = equals("subcommand2") >> wrap_help(option("option2"))
-    >>> p = subcommand1 | subcommand2
-    >>> p.parse_args("subcommand1", "-h")
-    usage: --option1 OPTION1
-    >>> p.parse_args("subcommand2", "-h")
-    usage: --option2 OPTION2
-    """
-    _help_parser: Parser[Sequence[A]] = help_parser(parser.usage, Sequence([]))
-
-    p = _help_parser >= (lambda _: parser)
-    return replace(p, usage=parser.usage, helps=parser.helps)
-
-
 def item(
     name: str,
     help_name: Optional[str] = None,
@@ -821,31 +964,6 @@ def nonpositional(*parsers: "Parser[Sequence[A]]") -> "Parser[Sequence[A]]":
     {'verbose': True, 'quiet': True}
     >>> p.parse_args("--quiet", "--verbose")  # reverse order also works
     {'quiet': True, 'verbose': True}
-
-    If alternatives or defaults appear among the arguments to `nonpositional`, you will probably want
-    to add `>>` followed by `done` (or another parser) after `nonpositional`. Otherwise,
-    the parser will not behave as expected:
-
-    >>> p = nonpositional(flag("verbose", default=False), flag("quiet"))
-    >>> p.parse_args("--quiet", "--verbose")  # you expect this to set verbose to True, but it doesn't
-    {'verbose': False, 'quiet': True}
-
-    Why is happening? There are two permutions:
-
-    - `flag("verbose", default=False) >> flag("quiet")` and
-    - `flag("quiet") >> flag("verbose", default=False)`
-
-    In our example, both permutations are actually succeeding. This first succeeds by falling
-    back to the default, and leaving the last word of the input, `--verbose`, unparsed.
-    Either interpretation is valid, and `nonpositional` returns one arbitrarily -- just not the one we expected.
-
-    Now let's add `>> done()` to the end:
-    >>> p = nonpositional(flag("verbose", default=False), flag("quiet")) >> done()
-
-    This ensures that the first permutation will fail because the leftover `--verbose` input will
-    cause the `done` parser to fail:
-    >>> p.parse_args("--quiet", "--verbose")
-    {'quiet': True, 'verbose': True}
     """
     if not parsers:
         return empty()
@@ -946,7 +1064,7 @@ def option(
 
     parser = Parser(f, usage=None, helps={})
     if type is not str:
-        parser = type_(type, parser)
+        parser = parser.type(type)
     if short and len(dest) > 1:
         parser2 = option(dest=dest, short=False, flag=f"-{dest[0]}", default=None)
         parser = parser | parser2
@@ -983,51 +1101,14 @@ def peak(
 
 
 def sat(
-    parser: Parser[A],
-    predicate: Callable[[A], bool],
-    on_fail: Callable[[A], ArgumentError],
-) -> Parser[A]:
-    """
-    Applies `parser`, applies a predicate to the result and fails if this returns false.
-
-    >>> p = sat(
-    ...     option("x", type=int).many(),
-    ...     lambda kvs: sum([kv.value for kv in kvs]) > 0,
-    ...     lambda x: ArgumentError(f"The values in {list(x)} must sum to more than 0."),
-    ... )
-    >>> p.parse_args("-x", "-1", "-x", "1")  # fails
-    usage: [-x X ...]
-    The values in [KeyValue(key='x', value=-1), KeyValue(key='x', value=1)] must sum to more than 0.
-
-    >>> p.parse_args("-x", "-1", "-x", "2")  # succeeds
-    {'x': 2}
-
-    Parameters
-    ----------
-    parser : Parser[A]
-        The parser to apply.
-    predicate : Callable[[A], bool]
-        The predicate to apply to the result of `parser`. `sat` fails if this predicate returns false.
-    on_fail : Callable[[A], ArgumentError]
-        A function producing an ArgumentError to return if the predicate fails.
-        Takes the output of `parser` as an argument.
-    """
-
-    def f(x: A) -> Result[A]:
-        return Result(NonemptyList(x) if predicate(x) else on_fail(x))
-
-    return apply(f, parser)
-
-
-def sat_item(
     predicate: Callable[[str], bool],
     on_fail: Callable[[str], ArgumentError],
     name: str,
 ) -> Parser[Sequence[KeyValue[str]]]:
     """
-    A wrapper around `sat` that uses `item` to parse the argument and just applies `predicate` to the value output by `item`.
+    A wrapper around `Parser.sat` that uses `item` to parse the argument and just applies `predicate` to the value output by `item`.
 
-    >>> p = sat_item(lambda x: len(x) == 1, lambda x: ArgumentError(f"'{x}' must have exactly one character."), "x")
+    >>> p = sat(lambda x: len(x) == 1, lambda x: ArgumentError(f"'{x}' must have exactly one character."), "x")
     >>> p.parse_args("a")  # succeeds
     {'x': 'a'}
     >>> p.parse_args("aa")  # fails
@@ -1053,7 +1134,7 @@ def sat_item(
         [kv] = parsed
         return on_fail(kv.value)
 
-    return sat(item(name), _predicate, _on_fail)
+    return item(name).sat(_predicate, _on_fail)
 
 
 def sat_peak(
@@ -1069,31 +1150,4 @@ def sat_peak(
         [kv] = parsed
         return on_fail(kv.value)
 
-    return sat(peak(name), _predicate, _on_fail)
-
-
-def type_(
-    f: Callable[[str], Any], parser: Parser[Sequence[KeyValue[str]]]
-) -> Parser[Sequence[KeyValue[Any]]]:
-    """
-    A wrapper around `apply` that simply applies `f` to the value of the most recently parsed input.
-    >>> p1 = option("x") >> option("y")
-    >>> p = type_(int, p1)
-    >>> p.parse_args("-x", "1", "-y", "2")  # converts "1" but not "2"
-    {'y': '2', 'x': 1}
-    """
-
-    def g(
-        kvs: Sequence[KeyValue[str]],
-    ) -> Result[Sequence[KeyValue[Any]]]:
-        head, *tail = kvs.get
-        try:
-            y = f(head.value)
-        except Exception as e:
-            usage = f"argument {head.value}: raised exception {e}"
-            return Result(ArgumentError(usage))
-        head = replace(head, value=y)
-        return Result.return_(Sequence([*tail, head]))
-
-    p = apply(g, parser)
-    return replace(p, usage=parser.usage, helps=parser.helps)
+    return peak(name).sat(_predicate, _on_fail)
